require 'capistrano/distribution/distributor'

# Avoid loading tasks definitions more than once.
# Fixes support for capistrano 3.5.0.
unless defined?(DISTRIBUTION_TASKS_LOADED) && DISTRIBUTION_TASKS_LOADED
  load File.expand_path('../tasks/distribution.cap', __FILE__)
  DISTRIBUTION_TASKS_LOADED = true
end

module Capistrano

##
# A list of distributors to run.
class Distribution
  ##
  # Creates the list of distributors to run based on the definition found in the
  # +:distribution+ key of _context_.  The value associated the +distribution+
  # can be 1 of 3 types:
  # * String
  # * Array of Arrays of distributor initialization arguments
  # * Array of distributor instances
  #
  # @param context [{#fetch, #repo_path, #release_path}] a Capistrano deployment
  #   context.
  #
  # @return [Distribution] an instance of this class.
  def initialize(context)
    @context = context
    distributor_list = context.fetch(:distribution)

    case distributor_list
    when String
      distributor_list = [[distributor_list]]
    when Array
      unless distributor_list.all? { |distributor| Array === distributor }
        distributor_list = [distributor_list]
      end
    end

    @distributors = distributor_list.map do |distributor|
      Distributor.create(context, distributor)
    end
  end

  ##
  # An identifier for a release.
  def release_id
    context.fetch(:release_id)
  end

  ##
  # Calls the #check method of each distributor in the list.
  def check
    distributors.all? { |distributor| distributor.check }
  end

  ##
  # Calls the #distribute method of each distributor in the list.
  def distribute
    distributors.each { |distributor| distributor.distribute }
  end

  private

  ##
  # The list of distributors.
  attr_reader :distributors

  ##
  # The Capistrano context object.
  attr_reader :context
end

end
