module Capistrano
class Distribution
module Distributor

##
# @abstract Subclass and override {#check} and {#distribute} to create a
#   distributor.
#
# An abstract distributor upon which all distributors should ultimately be
# based.
#
# See the existing concrete distributor definitions for examples.
class Abstract
  ##
  # @param context [(#test, #execute)] a Capistrano context used to run
  #   commands.
  # @param url [URI, String] a URL to be used for fetching the artifact to be
  #   distributed
  # @param opts [Hash] options to override default settings
  # @option opts [String] :target ('') a path within the release area to be the
  #   root of the distribution.
  def initialize(context, url, opts = {})
    @context = context
    @url = URI === url ? url : URI.parse(url)
    @target = opts.fetch(:target, '')

    @repo_id = Digest::SHA1.hexdigest(url.to_s)
  end

  ##
  # @abstract Override to provide a meaningful check for prerequisites during
  #   deployment.
  #
  # @return [Boolean] +true+ when prerequisites are met and +false+
  #   otherwise.
  def check
    false
  end

  ##
  # @abstract Override to provide meaningful distribution logic during
  #   deployment.
  #
  # @return [nil]
  #
  # @raise [exception] when distribution fails.
  def distribute
  end

  private

  ##
  # The source URL for the artifact to be distributed.
  attr_reader :url

  ##
  # The Capistrano context in which the distributor will operate.
  attr_reader :context

  ##
  # A unique identifier for the distributor for use under {#repo_path}.
  attr_reader :repo_id

  ##
  # A path relative to the release location in which to distribute the artifact.
  attr_reader :target

  ##
  # @return [Pathname] a path to the release location in which to distribute the
  #   artifact based on {#target}
  def release_path
    context.release_path.join(target)
  end

  ##
  # @return [Pathname] a path under the repo location to a unique workspace for
  #   the distributor
  def repo_path
    context.repo_path.join(repo_id)
  end
end

end
end
end
