require 'capistrano/distribution/distributor/abstract_git'

module Capistrano
class Distribution
module Distributor

##
# Distributes by pushing a local Git repository into a clone on each target host
# and then extracting the content from a revision to the release location.
class GitPush < AbstractGit
  ##
  # Always returns +true+ because this distributor does not pull content but
  # rather relies on the distribute step to push the content up.
  #
  # @return [true]
  def check
    true
  end

  ##
  # Prepares a Git repository if necessary, pushes the selected revision to the
  # Git repository, and finally extracts the revision to the release area.  The
  # Git repository is left in place in order to speed up future deployments by
  # avoiding the need to upload *all* revision history again.
  #
  # @return [nil]
  #
  # @raise [exception] when distribution fails.
  def distribute
    init
    push
    release
    nil
  end

  private

  ##
  # Initializes a bare Git repository in {#repo_path} if a repository does not
  # already exist there.
  #
  # @return [nil]
  #
  # @raise [exception] when repository creation fails.
  def init
    if context.test '[', '!', '-e', repo_path.join('HEAD'), ']'
      context.execute 'rm', '-rf', repo_path
      context.execute 'git', 'init', '--bare', repo_path
    end
    nil
  end

  ##
  # Pushes {#revision} and all its history to the Git repository within
  # {#repo_path}.
  #
  # @raise [exception] when the push to the repository fails.
  def push
    user = context.host.user
    hostname = context.host.hostname
    repo_path = self.repo_path
    revision = self.revision
    context.run_locally do
      execute 'git', 'push',
              "ssh://#{user}@#{hostname}/#{repo_path}",
              "+#{revision}:refs/tags/deploy"
    end
  end
end

end
end
end
